@@ -33,7 +33,6 @@
 #include <pan/filters/filter-aggregate.h>
 #include <pan/filters/filter-binary.h>
 #include <pan/filters/filter-cached.h>
-#include <pan/filters/filter-edit-ui.h>
 #include <pan/filters/filter-manager.h>
 #include <pan/filters/filter-new.h>
 #include <pan/filters/filter-phrase.h>
@@ -41,11 +40,12 @@
 #include <pan/filters/filter-saved.h>
 #include <pan/filters/filter-thread.h>
 #include <pan/filters/filter-top.h>
-#include <pan/filters/filter-ui.h>
 #include <pan/filters/filter-manager.h>
 
 #include <pan/article-list-toolbar-mediator.h> /* just for the util */
 #include <pan/article-toolbar.h>
+#include <pan/filter-ui.h>
+#include <pan/filter-edit-ui.h>
 #include <pan/globals.h>
 #include <pan/gui.h>
 #include <pan/util.h>
@@ -60,6 +60,8 @@
 #include <xpm/new_read.xpm>
 #include <xpm/old_unread.xpm>
 #include <xpm/watched.xpm>
+#include <xpm/by_me.xpm>
+#include <xpm/by_others.xpm>
 
 static GtkWidget * show_matches_button = NULL;
 static GtkWidget * show_subthread_button = NULL;
@@ -81,6 +83,8 @@ static GtkWidget * show_queued_button = NULL;
 static GtkWidget * show_idle_button = NULL;
 static GtkWidget * show_cached_button = NULL;
 static GtkWidget * show_non_cached_button = NULL;
+static GtkWidget * show_by_me_button = NULL;
+static GtkWidget * show_by_others_button = NULL;
 static GtkWidget * _custom_filters_menu = NULL;
 static GtkWidget * _no_customs_button = NULL;
 
@@ -172,6 +176,11 @@ get_filter_bits (void)
 	if (is_active (show_non_cached_button))
 	 	l |= STATE_FILTER_NOT_CACHED;
 
+	if (is_active (show_by_me_button))
+		l |= STATE_FILTER_MINE;
+	if (is_active (show_by_others_button))
+		l |= STATE_FILTER_NOT_MINE;
+
 	debug_exit ("get_filter_bits");
 	return l;
 }
@@ -252,6 +261,10 @@ update_filter_bit_menu_buttons (gulong l)
 	                                l & STATE_FILTER_CACHED);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_non_cached_button),
 	                                l & STATE_FILTER_NOT_CACHED);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_by_me_button),
+	                                l & STATE_FILTER_MINE);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(show_by_others_button),
+	                                l & STATE_FILTER_NOT_MINE);
 
 	ignore_toggle = FALSE;
 	pan_unlock ();
@@ -295,6 +308,9 @@ sensitize_builtin_filters (gboolean b)
 	gtk_widget_set_sensitive (show_cached_button, b);
 	gtk_widget_set_sensitive (show_non_cached_button, b);
 
+	gtk_widget_set_sensitive (show_by_me_button, b);
+	gtk_widget_set_sensitive (show_by_others_button, b);
+
 	debug_exit ("sensitize_builtin_filters");
 }
 
@@ -475,7 +491,7 @@ update_custom_menus (void)
 
 	/* create a hashtable for finding the menu button */
 	if (_name_to_menu != NULL) {
-		// FIXME: g_free the key strings
+		/* FIXME: g_free the key strings */
 		g_hash_table_destroy (_name_to_menu);
 	}
 	_name_to_menu = g_hash_table_new (g_str_hash, g_str_equal);
@@ -705,16 +721,6 @@ build_popdown_filter_menu (void)
 	                                 new_read_xpm, NULL,
 	                                 &show_read_button));
 	gtk_menu_append (m, create_menu_separator());
-	gtk_menu_append (m, bit_menu_new(_("Show Watched Threads"),
-	                                 watched_xpm, NULL,
-	                                 &show_watched_button));
-	gtk_menu_append (m, bit_menu_new(_("Show Ignored Threads"),
-	                                 NULL, GNOME_STOCK_MENU_TRASH_FULL,
-	                                 &show_killfile_button));
-	gtk_menu_append (m, bit_menu_new(_("Show Threads Neither Watched nor Ignored"),
-	                                 NULL, NULL,
-	                                 &show_normal_rank_button));
-	gtk_menu_append (m, create_menu_separator());
 	gtk_menu_append (m, bit_menu_new(_("Show Complete Binary Articles"),
 	                                 binary_complete_xpm, NULL,
 	                                 &show_complete_binary_button));
@@ -741,7 +747,24 @@ build_popdown_filter_menu (void)
 	gtk_menu_append (m, bit_menu_new(_("Show Articles which are not Cached Locally"),
 	                                 NULL, NULL,
 	                                 &show_non_cached_button));
-
+	gtk_menu_append (m, create_menu_separator());
+	gtk_menu_append (m, bit_menu_new(_("Show Articles posted by Me"),
+	                                 by_me_xpm, NULL,
+                                         &show_by_me_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Articles posted by Others"),
+	                                 by_others_xpm, NULL,
+	                                 &show_by_others_button));
+	gtk_menu_append (m, create_menu_separator());
+	gtk_menu_append (m, bit_menu_new(_("Show Watched Threads"),
+	                                 watched_xpm, NULL,
+	                                 &show_watched_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Ignored Threads"),
+	                                 NULL, GNOME_STOCK_MENU_TRASH_FULL,
+	                                 &show_killfile_button));
+	gtk_menu_append (m, bit_menu_new(_("Show Threads Neither Watched nor Ignored"),
+	                                 NULL, NULL,
+	                                 &show_normal_rank_button));
+	
 	/* add the submenu onto a dropdown button */
 	w = gtk_menu_item_new ();
 	gtk_container_add (GTK_CONTAINER(w), gnome_pixmap_new_from_xpm_d (filter_xpm));
@@ -844,11 +867,13 @@ article_toolbar_new (void)
 		int i;
 		ArticleToolbarOptionMenuStruct foo[] = {
 			{NULL, FILTER_PHRASE_SUBJECT},
-			{NULL, FILTER_PHRASE_AUTHOR}
+			{NULL, FILTER_PHRASE_AUTHOR},
+			{NULL, FILTER_PHRASE_MESSAGE_ID}
 		};
 		const int row_qty = sizeof(foo) / sizeof(foo[0]);
 		foo[0].name = _("Subject");
 		foo[1].name = _("Author");
+		foo[2].name = _("Message-ID");
 		for (i=0; i<row_qty; ++i) {
 			GtkWidget * item = gtk_menu_item_new_with_label (foo[i].name);
 			gtk_signal_connect (GTK_OBJECT(item), "activate",
@@ -865,6 +890,8 @@ article_toolbar_new (void)
 	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
 	_filter_phrase_entry = w = gnome_entry_new ("Article Toolbar Filter");
 	gnome_entry_set_max_saved (GNOME_ENTRY(w), 10);
+        gtk_signal_connect (GTK_OBJECT (gnome_entry_gtk_entry(GNOME_ENTRY(w))), "focus_out_event",
+	                    GTK_SIGNAL_FUNC(article_phrase_activate_cb), NULL);
 	gtk_signal_connect (GTK_OBJECT (gnome_entry_gtk_entry(GNOME_ENTRY(w))), "activate",
 	                    GTK_SIGNAL_FUNC(article_phrase_activate_cb), NULL);
 	gtk_signal_connect (GTK_OBJECT(GTK_COMBO(w)->popwin), "button_press_event",
