@@ -26,7 +26,6 @@
 #include <glib.h>
 
 #include <pan/base/article.h>
-#include <pan/base/status-item.h>
 
 typedef enum
 {
@@ -37,7 +36,7 @@ typedef enum
 	ARTICLE_SORT_MSG_ID         = 5,
 	ARTICLE_SORT_READ_STATE     = 6,
 	ARTICLE_SORT_ACTION_STATE   = 7,
-	ARTICLE_SORT_TYPES_QTY      = 7,
+	ARTICLE_SORT_TYPES_QTY      = 7
 }
 ArticleSortType;
 
@@ -46,10 +45,7 @@ void sort_articles   (Article      ** buf,
                       gint            sort_type,
                       gboolean        ascending);
 
-/* this calls queue_item_emit_next_step list_len times. */
-void thread_articles (GPtrArray     * articles,
-                      StatusItem    * status);
+void thread_articles (GPtrArray     * articles);
 
-void check_multipart_articles (GPtrArray * articles);
 
 #endif /* __ARTICLE_THREAD_H__ */
