@@ -24,11 +24,14 @@
 
 #include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>
 
 #include <glib.h>
 
+#include <gmime/memchunk.h>
+
 #include <pan/base/article.h>
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
@@ -74,7 +77,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 	g_free (path);
 
 	/* allocate the articles array */
-	pan_warn_if_fail (group->_articles == NULL);
+	pan_warn_if_fail (group->_articles==NULL);
 	group_get_articles (group);
 
 	if (dat!=NULL && idx!=NULL)
@@ -82,7 +85,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 		const gchar * march = idx->data;
 		const glong version = get_next_token_int (march, '\n', &march);
 
-		if (version==1 || version==2 || version==3 || version==4)
+		if (version==1 || version==2 || version==3 || version==4 || version==5)
 		{
 			int i;
 			long l;
@@ -134,8 +137,15 @@ file_headers_load_group (Group * group, StatusItem * status)
 				/* references */
 				l = get_next_token_long (march, '\n', &march);
 				if (0<=l && l<dat->len)
-					article_init_header (a, HEADER_REFERENCES, dat->data+l, 0);
-				/* xrefs added in version 4 */ if (version>=4) { l = get_next_token_long (march, '\n', &march); if (0<=l && l<dat->len) a->xref = dat->data + l; }
+					a->references = dat->data + l;
+
+				/* xrefs added in version 4 */
+				if (version>=4) {
+					l = get_next_token_long (march, '\n', &march);
+					if (0<=l && l<dat->len)
+						a->xref = dat->data + l;
+				}
+
 				/* numeric fields */
 				a->part           = (gint16) get_next_token_int (march, '\n', &march);
 				a->parts          = (gint16) get_next_token_int (march, '\n', &march);
@@ -145,6 +155,16 @@ file_headers_load_group (Group * group, StatusItem * status)
 				a->date           = (time_t) get_next_token_ulong (march, '\n', &march);
 				a->number         = (gulong) get_next_token_ulong (march, '\n', &march);
 
+				if (1) {
+					/* an article is new if it's flagged as new AND it's unread */
+					gboolean b = FALSE;
+					if (version >= 5)
+						b = get_next_token_int (march, '\n', &march) != 0;
+					if (b)
+						b = !newsrc_is_article_read (group_get_newsrc(group), a->number);
+					a->is_new = b;
+				}
+
 				/* extra headers */
 				extra_header_qty  = get_next_token_int (march, '\n', &march);
 				for (j=0; j<extra_header_qty; ++j)
@@ -181,7 +201,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 					group_get_readable_name(group));
 			}
 
-			group_add_articles_remove_unused (group, addme, status);
+			group_init_articles (group, addme, status);
 			group_set_article_qty (group, addme->len);
 			group->articles_dirty = purged_article_count!=0;
 			success = TRUE;
@@ -230,6 +250,7 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 	gchar * text;
 	const gchar * march;
 	GPtrArray * articles;
+	GString * msg;
 	debug_enter ("file_headers_load_folder");
 
 	g_return_val_if_fail (folder!=NULL, FALSE);
@@ -255,19 +276,18 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 	i = 0;
 	march = text;
 	articles = g_ptr_array_new ();
+	msg = g_string_sized_new (2048);
 	while (march != NULL)
 	{
 		gchar * from = NULL;
-		gchar * msg = NULL;
 
 		/* parse the next mbox message */
-		march = mbox_get_next_message (march, &from, &msg);
-		if (msg != NULL)
+		march = mbox_get_next_message (march, &from, msg);
+		if (msg->len != 0)
 		{
 			Article * a = article_new (folder);
 			a->number = ++i;
-			article_set_from_raw_message (a, msg);
-			g_free (msg);
+			article_set_from_raw_message (a, msg->str);
 
 			/* keep the good ones */
 			if (article_is_valid (a))
@@ -276,10 +296,11 @@ file_headers_load_folder (Group * folder, StatusItem * status)
 
 		g_free (from);
 	}
+	g_string_free (msg, TRUE);
 
 	/* if we've got articles then add them; otherwise, clean up */
 	if (articles->len != 0)
-		group_add_articles_remove_unused (folder, articles, status);
+		group_init_articles (folder, articles, status);
 	else {
 		g_hash_table_destroy (folder->_articles);
 		folder->_articles = NULL;
@@ -347,7 +368,7 @@ file_headers_load (Group * group, StatusItem * status)
  */
 long
 get_string_offset (GHashTable   * hash,
-                   GMemChunk    * long_chunk,
+                   MemChunk     * long_chunk,
                    const gchar  * str,
                    FILE         * fp,
                    long         * pos)
@@ -373,7 +394,7 @@ get_string_offset (GHashTable   * hash,
 	}
 	else /* first time; remember where we kept it */
 	{
-		glong * plong = g_chunk_new (glong, long_chunk);
+		glong * plong = (glong*) memchunk_alloc (long_chunk);
 		retval = *plong = *pos;
 		g_hash_table_insert (hash, (gpointer)str, plong);
 		*pos += fwrite (str, 1, strlen(str)+1, fp);
@@ -383,29 +404,47 @@ get_string_offset (GHashTable   * hash,
 }
 
 
-static void
+static gboolean
 file_headers_save_folder (Group * folder, StatusItem * status)
 {
-	FILE * fp;
-	gchar * fname;
-	gchar * fname_tmp;
+	FILE * fp = NULL;
+	gchar * fname = NULL;
+	gchar * fname_tmp = NULL;
+	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_folder");
 
-	g_return_if_fail (group_is_folder(folder));
+	/* sanity clause */
+	g_return_val_if_fail (folder!=NULL, FALSE);
+	g_return_val_if_fail (group_is_folder(folder), FALSE);
+
+	/* make sure the directory exists */
+	if (ok) {
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, folder->server->name);
+		if (!directory_check (pch)) {
+			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
+			ok = FALSE;
+		}
+		g_free (pch);
+	}
 
 	/* get the filenames & start writing to the outfile */
-	fname = get_mbox_filename (folder);
-	fname_tmp = g_strdup_printf ("%s.tmp", fname);
-	fp = fopen (fname_tmp, "w+");
+	if (ok) {
+		fname = get_mbox_filename (folder);
+		fname_tmp = g_strdup_printf ("%s.tmp", fname);
+		fp = fopen (fname_tmp, "w+");
+		if (fp == NULL) {
+			log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), fname_tmp);
+			ok = FALSE;
+		}
+	}
 
-	if (fp != NULL)
-	{
+	/* save the articles to the mbox */
+	if (ok) {
 		guint i = 0;
 		GPtrArray * array = group_get_article_array (folder);
 		GString * s = g_string_new (NULL);
 
-		for (i=0; i!=array->len; ++i)
-		{
+		for (i=0; i!=array->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(array,i));
 			gchar buf[32], *pch, *mbox;
 
@@ -433,28 +472,39 @@ file_headers_save_folder (Group * folder, StatusItem * status)
 
 			/* write the message in mbox format */
 			mbox = mbox_format_message (s->str, a->author_addr, a->date);
-			if (mbox != NULL)
-			{
-				fputs (mbox, fp);
+			if (mbox != NULL) {
+				if (fputs (mbox, fp) == EOF) {
+					ok = FALSE;
+					break;
+				}
 				g_free (mbox);
 			}
+
+			pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
 		}
 		fclose (fp);
 
-		/* wrote to tmp file okay... now make it permanent */
-		pan_rename (fname_tmp, fname);
-
 		/* cleanup */
 		g_ptr_array_free (array, TRUE);
 		g_string_free (s, TRUE);
 	}
 
+	/* wrote to tmp file okay... now make it permanent */
+	if (ok) {
+		if (!pan_rename (fname_tmp, fname)) {
+			log_add_va (LOG_ERROR, _("The group will not be saved -- can't move \"%s\" to \"%s\""),
+			fname_tmp, fname);
+			ok = FALSE;
+		}
+	}
+
 	g_free (fname);
 	g_free (fname_tmp);
 	debug_exit ("file_headers_save_folder");
+	return ok;
 }
 
-static void
+static gboolean
 file_headers_save_group (Group * group, StatusItem * status)
 {
 	guint i;
@@ -468,48 +518,65 @@ file_headers_save_group (Group * group, StatusItem * status)
 	GHashTable * hash_mail = NULL;
 	GHashTable * hash_refs = NULL;
 	GHashTable * hash_misc = NULL;
-	GMemChunk * long_chunk = NULL;
+	MemChunk * long_chunk = NULL;
 	GPtrArray * articles;
+	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_group");
 
-	g_return_if_fail (group!=NULL);
-	g_return_if_fail (group->server!=NULL);
-	g_return_if_fail (group->_articles!=NULL);
-	g_return_if_fail (is_nonempty_string(data_dir));
+	g_return_val_if_fail (group!=NULL, FALSE);
+	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (group->_articles!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	/* if nothing to save */
 	articles = group_get_article_array (group);
 	if (articles->len == 0) {
 		g_ptr_array_free (articles, TRUE);
 		file_headers_destroy (group);
-		return;
+		return FALSE;
 	}
 
 	/**
 	***  Save the Headers
 	**/
 
+	if (1) {
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, group->server->name);
+		if (!directory_check (pch))
+			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
+		g_free (pch);
+	}
+
+
 	/* open index file */
-	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
+		data_dir, G_DIR_SEPARATOR,
+		group->server->name, G_DIR_SEPARATOR,
+		group->name);
 	idx_fp = fopen (idx_path, "w+");
 	if (idx_fp == NULL) {
+		log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), idx_path);
 		g_free (idx_path);
-		return;
+		return FALSE;
 	}
 
 	/* open data file */
-	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp",
+		data_dir, G_DIR_SEPARATOR,
+		group->server->name, G_DIR_SEPARATOR,
+		group->name);
 	dat_fp = fopen (dat_path, "w+");
 	if (dat_fp == NULL) {
+		log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), dat_path);
 		fclose (idx_fp);
 		remove (idx_path);
 		g_free (idx_path);
 		g_free (dat_path);
-		return;
+		return FALSE;
 	}
 
 	/* Write DATBASE_VERSION */
-	fprintf (idx_fp, "4\n%ld\n", (long)articles->len);
+	fprintf (idx_fp, "5\n%ld\n", (long)articles->len);
 
 	/* Write the article information... */
 	pos = 0;
@@ -517,7 +584,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	hash_mail = g_hash_table_new (g_str_hash, g_str_equal);
 	hash_refs = g_hash_table_new (g_str_hash, g_str_equal);
 	hash_misc = g_hash_table_new (g_str_hash, g_str_equal);
-	long_chunk = g_mem_chunk_create (glong, 8192, G_ALLOC_ONLY);
+	long_chunk = memchunk_new (sizeof(glong), 4096, FALSE);
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index (articles, i));
@@ -542,15 +609,16 @@ file_headers_save_group (Group * group, StatusItem * status)
 			"%d\n"
 			"%d\n"
 			"%lu\n"
-			"%lu\n",
+			"%lu\n"
+			"%d\n",
 			id_idx, author_addr_idx, author_real_idx, subj_idx, refs_idx, xref_idx,
-			(int)a->part,
-			(int)a->parts,
+			(int)a->part, (int)a->parts,
 			(unsigned int)a->linecount,
 			(int)a->crosspost_qty,
 			(int)a->state,
 			(unsigned long)a->date,
-			(unsigned long)a->number);
+			(unsigned long)a->number,
+			(int)(a->is_new != 0));
 
 		/* make sure the extra headers are in pairs */
 		if ((extra_headers->len % 2) != 0) {
@@ -580,74 +648,117 @@ file_headers_save_group (Group * group, StatusItem * status)
 			fprintf (idx_fp, "%ld\n%ld\n", key_index, val_index);
 		}
 
+		pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
 		g_ptr_array_free (extra_headers, TRUE);
 	}
 	g_hash_table_destroy (hash_name); hash_name = NULL;
 	g_hash_table_destroy (hash_mail); hash_mail = NULL;
 	g_hash_table_destroy (hash_refs); hash_refs = NULL;
 	g_hash_table_destroy (hash_misc); hash_misc = NULL;
-	g_mem_chunk_destroy (long_chunk);
+	memchunk_destroy (long_chunk);
 
-	/* the write went okay; move the idx file over */
+	/* close the temp files */
 	fclose (idx_fp);
-	pch = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
-	pan_rename (idx_path, pch);
-	g_free (idx_path);
-	g_free (pch);
-
-	/* the write went okay; move the dat file over */
 	fclose (dat_fp);
-	pch = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
-	pan_rename (dat_path, pch);
-	g_free (dat_path);
-	g_free (pch);
+
+	ok = TRUE;
+
+	if (ok)
+	{
+		pch = g_strdup_printf ("%s%c%s%c%s.idx",
+			data_dir, G_DIR_SEPARATOR,
+			group->server->name, G_DIR_SEPARATOR,
+			group->name);
+		ok = pan_rename (idx_path, pch);
+		g_free (idx_path);
+		g_free (pch);
+	}
+
+	if (ok)
+	{
+		pch = g_strdup_printf ("%s%c%s%c%s.dat",
+			data_dir, G_DIR_SEPARATOR,
+			group->server->name, G_DIR_SEPARATOR,
+			group->name);
+		ok = pan_rename (dat_path, pch);
+		g_free (dat_path);
+		g_free (pch);
+	}
 
 	/* cleanup */
 	g_ptr_array_free (articles, TRUE);
 	debug_exit ("file_headers_save_group");
+	return ok;
+}
+
+void
+file_headers_save_noref (Group * group, StatusItem * status)
+{
+	static GStaticMutex _mutex = G_STATIC_MUTEX_INIT;
+
+	debug_enter ("file_headers_save_noref");
+	g_static_mutex_lock (&_mutex);
+
+	if (group->articles_dirty)
+	{
+		GPtrArray * articles;
+
+		/* since we're saving, mark dirty to FALSE */
+		group->articles_dirty = FALSE;
+
+		articles = group_get_article_array (group);
+		if (articles->len == 0) /* destroy the group */
+		{
+			file_headers_destroy (group);
+		}
+		else /* save the group */
+		{
+			struct timeval start;
+			struct timeval finish;
+			double diff;
+			gboolean ok = TRUE;
+
+			/* start the stopwatch */
+			gettimeofday (&start, NULL);
+
+			/* save the group */
+			ok = group_is_folder (group)
+				? file_headers_save_folder (group, status)
+				: file_headers_save_group (group, status);
+
+			if (!ok)
+				group->articles_dirty = TRUE;
+			else {
+				/* end the stopwatch */
+				gettimeofday (&finish, NULL);
+				diff = finish.tv_sec - start.tv_sec;
+				diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+				log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
+					articles->len,
+					group_get_readable_name (group),
+					diff,
+					articles->len/(fabs(diff)<0.001?0.001:diff));
+			}
+		}
+
+		/* cleanup */
+		g_ptr_array_free (articles, TRUE);
+	}
+
+	g_static_mutex_unlock (&_mutex);
+
+	debug_exit ("file_headers_save_noref");
 }
 void
 file_headers_save (Group * group, StatusItem * status)
 {
-	GPtrArray * articles;
-	struct timeval start;
-	struct timeval finish;
-	double diff;
 	debug_enter ("file_headers_save");
+	g_return_if_fail (group!=NULL);
 
-	gettimeofday (&start, NULL);
+	group_ref_articles (group, status);
+	file_headers_save_noref (group, status);
+	group_unref_articles (group, status);
 
-	/* trivial case #1: no change, so no need to save */
-	if (!group->articles_dirty)
-		return;
-
-	/* since we're saving, mark dirty to FALSE */
-	group->articles_dirty = FALSE;
-
-	articles = group_get_article_array (group);
-	if (articles->len == 0) /* destroy the group */
-	{
-		file_headers_destroy (group);
-		return;
-	}
-	else /* save the group */
-	{
-		if (group_is_folder (group))
-			file_headers_save_folder (group, status);
-		else
-			file_headers_save_group (group, status);
-		/* timing stats */
-		gettimeofday (&finish, NULL);
-		diff = finish.tv_sec - start.tv_sec;
-		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
-		log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
-			articles->len,
-			group_get_readable_name (group),
-			diff,
-			articles->len/(fabs(diff)<0.001?0.001:diff));
-	}
-
-	g_ptr_array_free (articles, TRUE);
 	debug_exit ("file_headers_save");
 }
 
