@@ -23,6 +23,8 @@
 
 #include <glib.h>
 
+#include <gmime/memchunk.h>
+
 #include <pan/base/newsrc.h>
 #include <pan/base/server.h>
 #include <pan/base/status-item.h>
@@ -60,7 +62,9 @@ enum
 	STATE_FILTER_NEW			= (1<<11),
 	STATE_FILTER_CACHED			= (1<<12),
 	STATE_FILTER_NOT_CACHED			= (1<<13),
-	STATE_FILTER_OLD                        = (1<<14)
+	STATE_FILTER_OLD                        = (1<<14),
+	STATE_FILTER_MINE                       = (1<<15),
+	STATE_FILTER_NOT_MINE                   = (1<<16)
 };
 
 #define GROUP(x) ((Group*)(x))
@@ -77,13 +81,14 @@ typedef struct _Group
 {
 	GMutex       * article_mutex;
 
+        guint          loaded_since_last_fetch  : 1;
         guint          articles_dirty           : 1;     /* do the articles need saving? */
-	guint          articles_threading_dirty : 1;
 	gchar          permission;                       /* (read only) 'y'=posting, 'n'=nopost, 'm'=moderated */
 	gint8          flags;                            /* (read only) new, subscribed, etc */
-	gint8          sort_style;                       /* (read only) hint for sorting in UI */
-	guint16        filter_bits;                      /* (read only) for filtering articles */
+	gint8          old_sort_style;                   /* (read only) hint for sorting in UI */
+	gint8          new_sort_style;                   /* (read only) hint for sorting in UI */
 	gulong         filter_show;                      /* (read only) for filtering articles */
+	guint32        filter_bits;                      /* (read only) for filtering articles */
 	gint32         article_qty;                      /* (read only) # of articles in group */
 	gint32         article_read_qty;                 /* (read only) # of read articles */
 	gulong         article_low;                      /* (read only) low article number */
@@ -101,10 +106,11 @@ typedef struct _Group
 
 	/* PRIVATE */
 	gint           _articles_refcount;
-	GMemChunk    * _article_chunk;                   /* Article objects */
+	MemChunk     * _article_chunk;                   /* Article objects */
 	Newsrc       * _newsrc;                          /* tracks read/unread */
 	Newsrc       * _purged;                          /* tracks deleted articles */
 	GHashTable   * _articles;                        /* articles */
+	GPtrArray    * _deleted_articles;                /* deleted articles */
 	gchar        * _one_big_chunk;                   /* used by loading local headers */
 }
 Group;
@@ -213,6 +219,10 @@ void          group_add_articles                  (Group            * group,
 					           GPtrArray        * used,
 					           GPtrArray        * ignored);
 
+void          group_init_articles                 (Group            * group,
+                                                   GPtrArray        * articles,
+                                                   StatusItem       * status_or_null);
+
 void          group_add_articles_remove_unused    (Group          * group,
                                                    GPtrArray        * articles,
                                                    StatusItem       * status_or_null);
@@ -238,9 +248,6 @@ void          group_expire_articles_not_in_range  (Group            * group,
                                                    gulong             low,
                                                    gulong             high);
 
-void          group_thread_if_needed              (Group            * group,
-                                                   StatusItem       * status);
-
 const gchar*  group_chunk_string                  (Group            * group,
                                                    const gchar      * chunkme,
                                                    gboolean           share);
@@ -301,19 +308,6 @@ PanCallback*  group_get_articles_removed_callback (void);
  */
 PanCallback*  group_get_groups_changed_callback (void);
 
-/**
- * @call_obj: Group*
- * @call_arg: non-NULL if all marked read, NULL if marked unread
- */
-PanCallback* group_get_group_marked_read_callback (void);
-
-/**
- * @call_obj: Group*
- * @call_arg: NULL
- */
-PanCallback* group_get_group_emptied_callback (void);
-
-
 
 
 #endif /* __GROUPS_H__ */
