@@ -24,12 +24,11 @@
 #include <string.h>
 
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-config.h>
 
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/server.h>
@@ -40,6 +39,9 @@ static GSList * serverlist = NULL;
 
 static Server * _active = NULL;
 
+const gchar * PAN_SENT;
+const gchar * PAN_SENDLATER;
+
 /**
 ***  Private Utilities
 **/
@@ -60,8 +62,10 @@ static void
 server_folders_load (void)
 {
 	Server * server;
+	GPtrArray * groups = g_ptr_array_new ();
 	debug_enter ("server_folders_load");
 
+
        	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
 	if (server == NULL) /* new user; create the folder server */
 	{
@@ -82,7 +86,7 @@ server_folders_load (void)
 		group->readable_name = g_strdup (_(PAN_SENDLATER));
 		group->description = g_strdup (_("Articles/Mail queued for posting via the \"Send Later\" button"));
 		group_set_is_folder (group, TRUE);
-		server_add_groups (server, &group, 1, NULL, NULL);
+		g_ptr_array_add (groups, group);
 		log_add (LOG_INFO, _("Created folder pan.sendlater"));
 	}
 
@@ -92,10 +96,14 @@ server_folders_load (void)
 		group->readable_name = g_strdup (_(PAN_SENT));
 		group->description = g_strdup (_("Articles/Mail posted before"));
 		group_set_is_folder (group, TRUE);
-		server_add_groups (server, &group, 1, NULL, NULL);
+		g_ptr_array_add (groups, group);
 		log_add (LOG_INFO, _("Created folder pan.sent"));
 	}
 
+	if (groups->len != 0)
+		server_add_groups (server, (Group**)groups->pdata, groups->len, NULL, NULL);
+
+	g_ptr_array_free (groups, TRUE);
 	debug_exit ("server_folders_load");
 }
 
@@ -106,16 +114,16 @@ server_fetch (const char *name)
 	gboolean has_section;
 	gchar * buf;
 
-	/* find out if we've got a gnome_config section for this server */
+	/* find out if we've got a pan_config section for this server */
 	buf = g_strdup_printf ("/Pan/%s/", name);
-	gnome_config_push_prefix (buf);
-	has_section = gnome_config_has_section (buf);
+	pan_config_push_prefix (buf);
+	has_section = pan_config_has_section (buf);
 	replace_gstr (&buf, NULL);
 
 	if (has_section)
 	{
 		server = server_new ();
-		server->address = gnome_config_get_string ("Address");
+		server->address = pan_config_get_string ("Address");
 		if (server->address == NULL)
 		{
 			pan_object_unref (PAN_OBJECT(server));
@@ -130,30 +138,30 @@ server_fetch (const char *name)
 		       	path = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, name);
 			directory_check (path);
 
-			/* load the server from gnome_config */
+			/* load the server from pan_config */
 			server->port =
-				gnome_config_get_int ("Port=119");
+				pan_config_get_int ("Port=119");
 			server->name =
 				g_strdup(name);
 			server->gen_msgid =
-				gnome_config_get_int ("Generate_Msgid=0");
+				pan_config_get_int ("Generate_Msgid=0");
 			server->need_auth =
-				gnome_config_get_int ("Auth_Required=0");
+				pan_config_get_int ("Auth_Required=0");
 			server->username =
-				gnome_config_get_string ("Username");
+				pan_config_get_string ("Username");
 			server->password =
-				gnome_config_get_string ("Password");
+				pan_config_get_string ("Password");
 			server->max_connections =
-				gnome_config_get_int ("Max_Connections=2");
+				pan_config_get_int ("Max_Connections=2");
 			server->idle_secs_before_timeout =
-				gnome_config_get_int ("Idle_Timeout_Secs=180");
+				pan_config_get_int ("Idle_Timeout_Secs=180");
 			server->online =
-				gnome_config_get_bool ("Online=TRUE");
+				pan_config_get_bool ("Online=TRUE");
 			server->reserve_connection_for_bodies =
-				gnome_config_get_bool ("Reserve_Reader_Connection=TRUE");
-			/* no good way to get a ulong from gnome_config? */
+				pan_config_get_bool ("Reserve_Reader_Connection=TRUE");
+			/* no good way to get a ulong from pan_config? */
 			server->last_newgroup_list_time = 1;
-			pch = gnome_config_get_string ("Last_Grouplist_Update");
+			pch = pan_config_get_string ("Last_Grouplist_Update");
 			if (is_nonempty_string(pch))
 				server->last_newgroup_list_time = strtoul (pch, NULL, 10);
 
@@ -162,7 +170,7 @@ server_fetch (const char *name)
 			g_free (pch);
 		}
 	}
-	gnome_config_pop_prefix ();
+	pan_config_pop_prefix ();
 
 	return server;
 }
@@ -175,7 +183,7 @@ static GSList*
 server_list_load (void)
 {
 	GSList * l = NULL;
-	gchar * p = gnome_config_get_string ("/Pan/Servers/Names"); /*commas*/
+	gchar * p = pan_config_get_string ("/Pan/Servers/Names"); /*commas*/
 
 	/* build all the servers in the list */
 	if (p != NULL)
@@ -304,7 +312,7 @@ get_default_server (void)
 	g_return_val_if_fail (serverlist!=NULL, NULL);
 
 	/* look in the gnome config for a default */
-	name = gnome_config_get_string ("/Pan/State/Server");
+	name = pan_config_get_string ("/Pan/State/Server");
 	if (is_nonempty_string (name))
 		retval = serverlist_get_named_server (name);
 	g_free (name);
@@ -350,6 +358,7 @@ serverlist_add_server (Server * server)
 	serverlist = g_slist_insert_sorted (serverlist,
 	                                    server,
 	                                    server_name_compare);
+	serverlist_save ();
 }
 
 void
@@ -366,89 +375,75 @@ serverlist_remove_server (Server * server)
 void
 serverlist_save (void)
 {
-	GString * buf = g_string_new (NULL);
 	GString * servers = g_string_new (NULL);
 	GSList *l = NULL;
 	gchar * p;
+	char buf[2048];
 
 	/* Out with the old */
-	p = gnome_config_get_string ("/Pan/Servers/Names");
+	p = pan_config_get_string ("/Pan/Servers/Names");
 	if (p != NULL) {
-		const char * pch = p;
-		while (get_next_token_g_str(pch, ',', &pch, buf)) {
-			g_string_prepend (buf, "/Pan/");
-			gnome_config_clean_section (buf->str);
+		int str_len = 0;
+		const char * str_begin = NULL;
+		const char * walk = p;
+		while (get_next_token_run (walk, ',', &walk, &str_begin, &str_len)) {
+			g_snprintf (buf, sizeof(buf), "/Pan/%*.*s", str_len, str_len, str_begin);
+			pan_config_clean_section (buf);
 		}
 		g_free (p);
 	}
-	gnome_config_sync ();
+	pan_config_sync ();
 
 	/* In with the new */
 	for (l=serverlist; l!=NULL; l=l->next)
 	{
-		char * pch;
+		char val[128];
 		Server *s = SERVER(l->data);
 
-		g_string_sprintf (buf, "/Pan/%s/Address", s->name);
-		gnome_config_set_string (buf->str, s->address);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Address", s->name);
+		pan_config_set_string (buf, s->address);
 
-		g_string_sprintf (buf, "/Pan/%s/Port", s->name );
-		gnome_config_set_int (buf->str, s->port);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Port", s->name );
+		pan_config_set_int (buf, s->port);
 
-		g_string_sprintf (buf, "/Pan/%s/Auth_Required", s->name);
-		gnome_config_set_int (buf->str, s->need_auth);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Auth_Required", s->name);
+		pan_config_set_int (buf, s->need_auth);
 
-		g_string_sprintf (buf, "/Pan/%s/Generate_Msgid", s->name);
-		gnome_config_set_int (buf->str, s->gen_msgid);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Generate_Msgid", s->name);
+		pan_config_set_int (buf, s->gen_msgid);
 
-		g_string_sprintf (buf, "/Pan/%s/Username", s->name);
-		gnome_config_set_string (buf->str, s->username);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Username", s->name);
+		pan_config_set_string (buf, s->username);
 
-		g_string_sprintf (buf, "/Pan/%s/Password", s->name);
-		gnome_config_set_string (buf->str, s->password);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Password", s->name);
+		pan_config_set_string (buf, s->password);
 
-		g_string_sprintf (buf, "/Pan/%s/Max_Connections", s->name);
-		gnome_config_set_int (buf->str, s->max_connections);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Max_Connections", s->name);
+		pan_config_set_int (buf, s->max_connections);
 
-		g_string_sprintf (buf, "/Pan/%s/Idle_Timeout_Secs", s->name);
-		gnome_config_set_int (buf->str, s->idle_secs_before_timeout);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Idle_Timeout_Secs", s->name);
+		pan_config_set_int (buf, s->idle_secs_before_timeout);
 
-		g_string_sprintf (buf, "/Pan/%s/Online", s->name);
-		gnome_config_set_bool (buf->str, s->online);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Online", s->name);
+		pan_config_set_bool (buf, s->online);
 
-		/* no good way to store a ulong in gnome_config? */
-		pch = g_strdup_printf ("%lu", s->last_newgroup_list_time);
-		g_string_sprintf (buf,"/Pan/%s/Last_Grouplist_Update", s->name);
-		gnome_config_set_string (buf->str, pch);
-		g_free (pch);
+		g_snprintf (buf, sizeof(buf),"/Pan/%s/Last_Grouplist_Update", s->name);
+		g_snprintf (val, sizeof(val), "%lu", s->last_newgroup_list_time);
+		pan_config_set_string (buf, val);
 
-		g_string_sprintf (buf, "/Pan/%s/Reserve_Reader_Connection", s->name);
-		gnome_config_set_bool (buf->str, s->reserve_connection_for_bodies);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Reserve_Reader_Connection", s->name);
+		pan_config_set_bool (buf, s->reserve_connection_for_bodies);
 
 		if (servers->len)
 			g_string_append_c (servers, ',');
 		g_string_append (servers, s->name);
 	}
 
-	gnome_config_set_string ("/Pan/Servers/Names", servers->str);
+	pan_config_set_string ("/Pan/Servers/Names", servers->str);
 	g_string_free (servers, TRUE);
-	g_string_free (buf, TRUE);
-	gnome_config_sync ();
+	pan_config_sync ();
 }
 
-#if 0
-void
-serverlist_activate_default_server (void)
-{
-	Server * s = get_default_server ();
-	if (s != NULL) {
-		_active = s;
-		fire_server_activated (s);
-	}
-}
-#endif
-
-
 /**
 ***  Server Activated
 **/
