@@ -57,6 +57,8 @@
 #include <pan/text.h> /* for colors of fg, bg, and quoted text */
 #include <pan/util.h>
 
+/* #define ATTACHMENTS */
+
 typedef struct
 {
 	GtkWidget * window;
@@ -74,11 +76,18 @@ typedef struct
         GtkWidget * reply_to;
 	GtkWidget * body;
 
+#ifdef ATTACHMENTS
 	/* Attachment */
-	GtkWidget * filename;
-	GtkWidget * lines;
-	GtkWidget * lines_per_part;
-	GtkWidget * parts;
+	GtkWidget * file_clist;
+	GtkWidget * file_add_button;
+	GtkWidget * file_remove_button;
+	GtkWidget * attachment_lines_per_part_sb;
+	GtkWidget * single_part_rb;
+	GtkWidget * multi_part_rb;
+	GtkWidget * encoding_mime_tb;
+	GtkWidget * encoding_uuenc_tb;
+	GtkWidget * encoding_yenc_tb;
+#endif
 
 	Article * article;
 	ComposeType type;
@@ -109,11 +118,6 @@ static void message_window_close           (GtkWidget*, Compose*);
 static void compose_ext_editor_cb          (GtkWidget*, Compose*);
 static void identities_changed_cb          (gpointer, gpointer, gpointer);
 
-/* Not ready -- Matt
-static void attachment_file_selector       (GtkWidget*, Compose*);
-static GtkWidget * attachment_page         (Compose*);
-static void attachments_lines_per_part_changed (GtkWidget*, Compose*);
-*/
 static GtkWidget * extra_headers_page      (Compose*);
 static GtkWidget * create_body_pane_nolock (Compose*);
 
@@ -359,7 +363,7 @@ create_reply_body (Article * article)
 	gchar * pch;
 
 	/* body */
-	pch = g_strdup (article_get_header (article, PAN_REPLY_PORTION));
+	pch = g_strdup (article_get_extra_header (article, PAN_REPLY_PORTION));
 	marked = pch != NULL;
 	if (pch == NULL)
 		pch = article_get_body (article);
@@ -432,7 +436,7 @@ rot13_cb (GtkWidget * widget, Compose * mw)
 	if (end > start)
 	{
 		gchar * str = gtk_editable_get_chars (e, start, end);
-		replace_gstr (&str, rot13(str));
+		rot13_inplace (str);
 		gtk_editable_delete_text (e, start, end);
 		gtk_editable_insert_text (e, str, strlen(str), &start);
 	}
@@ -521,70 +525,326 @@ update_title_with_subject (GtkWidget * widget, Compose * mw)
 	                      gtk_entry_get_text (GTK_ENTRY (widget)));
 }
 
-/* 
+#ifdef ATTACHMENTS
+
+/***
+****
+****   ATTACHMENTS
+****
+***/
+
+typedef enum
+{
+	ENCODING_BASE64,
+	ENCODING_UUENC,
+	ENCODING_YENC
+}
+PanEncoding;
+
+#if 0
+static PanEncoding
+get_encoding_type (Compose * mw)
+{
+	PanEncoding retval;
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->encoding_mime_tb)))
+		retval = ENCODING_BASE64;
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->encoding_uuenc_tb)))
+		retval = ENCODING_UUENC;
+	else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->encoding_yenc_tb)))
+		retval = ENCODING_YENC;
+	else
+		pan_warn_if_reached ();
+
+	return retval;
+}
+#endif
+
+static gulong
+get_estimated_encoded_size (gulong raw_size, PanEncoding encoding)
+{
+	switch (encoding) {
+		case ENCODING_BASE64: raw_size *= 1.33; break;
+		case ENCODING_UUENC: raw_size *= 1.35; break;
+		case ENCODING_YENC: raw_size *= 1.03; break;
+	}
+				    
+	return raw_size;
+}
+
+static void
+refresh_attachment_page (Compose * mw)
+{
+	size_t raw_size = 0;
+	size_t lines_per_part = 0;
+	GString * gstr = g_string_new (NULL);
+
+	/* get lines-per-part */
+	lines_per_part = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(mw->attachment_lines_per_part_sb));
+
+	/* get the unencoded size */
+	if (1) {
+		gint i;
+		GtkCList * clist = GTK_CLIST(mw->file_clist);
+		for (i=0; i<clist->rows; ++i) {
+			const gchar * filename = gtk_clist_get_row_data (clist, i);
+			raw_size += get_filesize (filename);
+		}
+	}
+
+	/* update mime label */
+	if (1) {
+		gulong encoded_size = get_estimated_encoded_size (raw_size, ENCODING_BASE64);
+		gulong lines = encoded_size / 45;
+		gulong parts = (lines/lines_per_part) + 1;
+
+		g_string_assign (gstr, _("Mime BASE64 Encoding - Single Articles Only "));
+		if (lines!=0 && parts==1)
+			g_string_sprintfa (gstr, _("(%lu lines in 1 article)"), lines);
+		else if (lines!=0)
+			g_string_sprintfa (gstr, _("(%lu lines in %lu articles)"), lines, parts);
+		gtk_label_set_text (GTK_LABEL(GTK_BIN(mw->encoding_mime_tb)->child), gstr->str);
+
+		if (parts>1 && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->encoding_mime_tb)))
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->encoding_uuenc_tb), TRUE);
+		gtk_widget_set_sensitive (mw->encoding_mime_tb, parts<2);
+	}
+
+	/* update uuenc label */
+	if (1) {
+		gulong encoded_size = get_estimated_encoded_size (raw_size, ENCODING_UUENC);
+		gulong lines = encoded_size / 45;
+		gulong parts = (lines/lines_per_part) + 1;
+
+		g_string_assign (gstr, _("UUEncoding - Universally Accepted "));
+		if (lines!=0 && parts==1)
+			g_string_sprintfa (gstr, _("(%lu lines in 1 article)"), lines);
+		else if (lines!=0)
+			g_string_sprintfa (gstr, _("(%lu lines in %lu articles)"), lines, parts);
+
+		gtk_label_set_text (GTK_LABEL(GTK_BIN(mw->encoding_uuenc_tb)->child), gstr->str);
+	}
+
+	/* update yenc label */
+	if (1) {
+		gulong encoded_size = get_estimated_encoded_size (raw_size, ENCODING_YENC);
+		gulong lines = encoded_size / 45;
+		gulong parts = (lines/lines_per_part) + 1;
+
+		g_string_assign (gstr, _("Yenc - Smaller files, less universal "));
+		if (lines!=0 && parts==1)
+			g_string_sprintfa (gstr, _("(%lu lines in 1 article)"), lines);
+		else if (lines!=0)
+			g_string_sprintfa (gstr, _("(%lu lines in %lu articles)"), lines, parts);
+		gtk_label_set_text (GTK_LABEL(GTK_BIN(mw->encoding_yenc_tb)->child), gstr->str);
+	}
+}
+
+static void
+attachments_lines_per_part_changed (GtkWidget * w, Compose * mw)
+{
+	refresh_attachment_page (mw);
+}
+
+static void
+attachments_page_file_selected (GtkWidget * w, GtkWidget * f)
+{
+	gint row;
+	char * text[2];
+	char line_buf[32];
+	char * selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(f));
+	Compose * mw = gtk_object_get_user_data (GTK_OBJECT(f));
+	GtkCList * clist = GTK_CLIST(mw->file_clist);
+
+	/* insert the file */
+	g_snprintf (line_buf, sizeof(line_buf), "%lu", (gulong)(get_filesize(selected_filename)/1024));
+	text[0] = selected_filename;
+	text[1] = line_buf;
+	row = gtk_clist_insert (clist, -1, text);
+	gtk_clist_set_row_data (clist, row, g_strdup(selected_filename));
+
+	/* update the line counts */
+	refresh_attachment_page (mw);
+}
+
+static void
+attachment_add_button_clicked_cb (GtkButton * button,
+                                  gpointer user_data)
+{
+	Compose * mw = (Compose*) user_data;
+	GtkWidget * w;
+
+	w = gtk_file_selection_new(_("Select the file to attach."));
+   	gtk_object_set_user_data (GTK_OBJECT(w), mw);
+	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(w)->ok_button),
+	                    "clicked", GTK_SIGNAL_FUNC (attachments_page_file_selected), w);
+    	gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(w)->ok_button),
+	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT(w));
+	gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(w)->cancel_button),
+	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT(w));
+    	gtk_widget_show (w);
+}
+
+static void
+attachment_remove_button_clicked_cb (GtkButton * button,
+                                     gpointer user_data)
+{
+	GList * l;
+	Compose * mw = (Compose*) user_data;
+	GtkCList * clist = GTK_CLIST (mw->file_clist);
+
+	l = clist->selection;
+	if (l != NULL)
+	{
+		/* remove the file */
+		gint row = GPOINTER_TO_INT (l->data);
+		gchar * filename = gtk_clist_get_row_data (clist, row);
+		g_free (filename);
+		gtk_clist_remove (clist, row);
+
+		/* update the line counts */
+		refresh_attachment_page (mw);
+	}
+}
+
+static void
+attachment_clist_selection_changed_cb (GtkCList * clist,
+                                       gint row,
+                                       gint column,
+                                       GdkEventButton * event,
+                                       gpointer user_data)
+{
+	Compose * mw = (Compose*) user_data;
+	gboolean has_selection = clist->selection != NULL;
+	gtk_widget_set_sensitive (mw->file_remove_button, has_selection);
+}
+
 static GtkWidget *
 attachment_page (Compose *mw)
 {
-	GtkWidget *table;
-	GtkWidget *w;
-	GtkObject *a;
+	char * titles[2];
+	GtkAdjustment * a;
+	GtkWidget * w;
+	GtkWidget * h;
+	GtkWidget * v;
+	GtkWidget * p;
+	GtkWidget * frame;
+	GtkWidget * top;
 
-	table = gtk_table_new (6, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
-	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
+	top = gtk_vbox_new (FALSE, GNOME_PAD);
 
-	w = gtk_label_new (_("Filename:"));
-	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 0, 1,
-                    GTK_FILL, GTK_FILL, GNOME_PAD, 0);
 
-	mw->filename = gtk_entry_new ();
-	gtk_table_attach (GTK_TABLE (table), mw->filename, 1, 5, 0, 1,
-                    GTK_EXPAND | GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	/**
+	***  Files Frame
+	**/
 
-	w = gtk_button_new_with_label (_("Select File"));
-	gtk_signal_connect (GTK_OBJECT (w), "clicked",
-                      GTK_SIGNAL_FUNC (attachment_file_selector), mw);
-	gtk_table_attach (GTK_TABLE (table), w, 5, 6, 0, 1,
-                    GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	frame = gtk_frame_new (_("Files to Attach"));
+	gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD_SMALL);
 
-	w = gtk_label_new (_("Lines per Part:"));
-	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 1, 2,
-                    GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	v = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER(v), GNOME_PAD);
+	gtk_container_add (GTK_CONTAINER(frame), v);
+	gtk_box_pack_start (GTK_BOX(top), frame, TRUE, TRUE, 0);
 
-	a = gtk_adjustment_new (7000, 100, 15000, 25, 10, 10);
-	mw->lines_per_part = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0);
-	gtk_table_attach (GTK_TABLE (table), mw->lines_per_part, 1, 2, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_FILL, GNOME_PAD, 0);
- 	gtk_signal_connect (GTK_OBJECT (a), "value_changed",
-                           GTK_SIGNAL_FUNC (attachments_lines_per_part_changed),
-                           (gpointer) mw);
+	/* Files clist */
+	titles[0] = _("Filename");
+	titles[1] = _("Kilobytes");
+	mw->file_clist = w = gtk_clist_new_with_titles (2, titles);
+	gtk_widget_set_usize (GTK_WIDGET(w), -1, 160);
+	gtk_clist_set_column_width (GTK_CLIST(w), 0, 400);
+	gtk_clist_set_column_width (GTK_CLIST(w), 1, 80);
+	gtk_signal_connect (GTK_OBJECT(w), "select_row",
+	                    GTK_SIGNAL_FUNC (attachment_clist_selection_changed_cb), mw);
+	gtk_signal_connect (GTK_OBJECT(w), "unselect_row",
+	                    GTK_SIGNAL_FUNC (attachment_clist_selection_changed_cb), mw);
+        w = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
+                GTK_POLICY_AUTOMATIC,
+                GTK_POLICY_AUTOMATIC);
+        gtk_container_add (GTK_CONTAINER(w), mw->file_clist);
+	gtk_box_pack_start (GTK_BOX(v), w, TRUE, TRUE, GNOME_PAD_SMALL);
 
-	w = gtk_label_new (_("Lines:"));
-	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), w, 2, 3, 1, 2,
-                    GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	h = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(v), h, FALSE, FALSE, 0);
 
-	mw->lines = gtk_label_new (_("0"));
-	gtk_misc_set_alignment (GTK_MISC (mw->lines), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), mw->lines, 3, 4, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	/* "Add" button */
+	mw->file_add_button = w = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
+	p = gnome_stock_pixmap_widget_new (mw->window, GNOME_STOCK_PIXMAP_ATTACH);
+	gtk_signal_connect (GTK_OBJECT(w), "clicked", attachment_add_button_clicked_cb, mw);
+	gtk_container_add (GTK_CONTAINER(w), p);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
+		_("Add a File to the Attachment List"), "");
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Parts:"));
-	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), w, 4, 5, 1, 2,
-                    GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	/* "Remove" button */
+	mw->file_remove_button = w = gtk_button_new ();
+	gtk_widget_set_sensitive (w, FALSE);
+	gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
+	p = gnome_stock_pixmap_widget_new (mw->window, GNOME_STOCK_PIXMAP_CLEAR);
+	gtk_signal_connect (GTK_OBJECT(w), "clicked",
+	                    attachment_remove_button_clicked_cb, mw);
+	gtk_container_add (GTK_CONTAINER(w), p);
+	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
+		_("Remove a File from the Attachment List"), "");
+	gtk_box_pack_end (GTK_BOX(h), w, FALSE, FALSE, 0);
 
-	mw->parts = gtk_label_new (_("0"));
-	gtk_misc_set_alignment (GTK_MISC (mw->parts), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), mw->parts, 5, 6, 1, 2,
-                    GTK_EXPAND | GTK_FILL, GTK_FILL, GNOME_PAD, 0);
+	/**
+	***  Parts Frame
+	**/
 
-	return table;
+	frame = gtk_frame_new (_("Parts"));
+	gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(top), frame, FALSE, FALSE, 0);
+
+	h = gtk_hbox_new (FALSE, GNOME_PAD);
+	gtk_container_set_border_width (GTK_CONTAINER(h), GNOME_PAD_BIG);
+	gtk_container_add (GTK_CONTAINER(frame), h);
+
+	w = gtk_label_new (_("Lines Per Article:"));
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+	a = GTK_ADJUSTMENT(gtk_adjustment_new (5000, 100, 15000, 100, 10, 10));
+	mw->attachment_lines_per_part_sb = w = gtk_spin_button_new (a, 0, 0);
+ 	gtk_signal_connect (GTK_OBJECT(a), "value_changed",
+	                    GTK_SIGNAL_FUNC (attachments_lines_per_part_changed), mw);
+	gtk_widget_set_usize (GTK_WIDGET(w), 70, 0);
+	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+
+
+	/**
+	***  Encoding Frame
+	**/
+
+	frame = gtk_frame_new (_("Encoding"));
+	gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(top), frame, FALSE, FALSE, 0);
+
+	v = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER(v), GNOME_PAD_BIG);
+	gtk_container_add (GTK_CONTAINER(frame), v);
+
+	w = gtk_radio_button_new_with_label (NULL, _("UUEncoded (universally accepted)"));
+	mw->encoding_uuenc_tb = w;
+	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
+	w = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON(w), _("Yenc (30 percent smaller than UUEnc but less universal)"));
+	mw->encoding_yenc_tb = w;
+	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
+	w = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON(w), _("Mime (single-part posts only)"));
+	mw->encoding_mime_tb = w;
+	gtk_box_pack_start (GTK_BOX(v), w, FALSE, FALSE, 0);
+
+
+	refresh_attachment_page (mw);
+	return top;
 }
-*/
+
+#endif
+
+/***
+****
+****   EXTRA HEADERS
+****
+***/
 
 static GtkWidget *
 extra_headers_page (Compose * mw)
@@ -783,14 +1043,13 @@ create_post_info_pane (Compose *mw)
 	notebook = gtk_notebook_new ();
 
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), main_page,
-			gtk_label_new (_("Message")));
+	                          gtk_label_new (_("Message")));
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), extra_headers_page(mw),
-			gtk_label_new (_("More Headers")));
-
-/* This isn't ready for the stable release -- Matt
- * gtk_notebook_append_page (GTK_NOTEBOOK (notebook), attachment_page(mw),
-			gtk_label_new (_("Attachments")));
-*/
+	                          gtk_label_new (_("More Headers")));
+#ifdef ATTACHMENTS
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), attachment_page(mw),
+	                          gtk_label_new (_("Attachments")));
+#endif
 
 	pan_unlock();
 
@@ -859,9 +1118,11 @@ article_gen_attribution_string (const Article * a, const gchar * attribution_lin
 	g_free (pch);
 
 	/* substitutions: author */
-	pch = a==NULL ? g_strdup("") : article_get_author_str (a);
-	pan_g_string_replace (gstr, "%a", pch);
-	g_free (pch);
+	if (1) {
+		gchar author[512] = { '\0' };
+		article_get_author_str (a, author, sizeof(author));
+		pan_g_string_replace (gstr, "%a", author);
+	}
 
 	/* substitutions: author name */
 	*author = '\0';
@@ -982,7 +1243,6 @@ append_signature (GtkWidget * body, const gchar *sig_file)
 {
 	gchar * sig = NULL;
 	gchar * pch;
-	gchar * text;
 
 	g_return_if_fail (body!=NULL);
 	g_return_if_fail (sig_file!=NULL);
@@ -990,14 +1250,16 @@ append_signature (GtkWidget * body, const gchar *sig_file)
 	/* since we're applying a signature, remove any signature anyway */
 	remove_signature (body);
 
-	text = gtk_editable_get_chars (GTK_EDITABLE (body), 0, -1);
-
         /* load the signature */	
-	pch = g_strdup(sig_file);
+	pch = g_strdup (sig_file);
+	g_strstrip (pch);
 	if (is_nonempty_string (pch))
 	{
 		GArray * array;
-		replace_gstr (&pch, pan_substitute (pch,"~",g_get_home_dir()));
+
+		if (*pch=='~')
+			replace_gstr (&pch, g_strconcat (g_get_home_dir(), pch+1, NULL));
+
 		array = read_file (pch);
 		if (array != NULL) {
 			sig = array->data;
@@ -1010,6 +1272,7 @@ append_signature (GtkWidget * body, const gchar *sig_file)
 	if (sig!=NULL)
 	{
 		gchar * out;
+		gchar * text = gtk_editable_get_chars (GTK_EDITABLE (body), 0, -1);
 
 		/* we insert "-- " if it's not already there */
 		gboolean delimiter_exists = (
@@ -1025,9 +1288,8 @@ append_signature (GtkWidget * body, const gchar *sig_file)
 
 		g_free (out);
 		g_free (sig);
+		g_free (text);
 	}
-
-	g_free (text);
 }
 
 /***
@@ -1263,7 +1525,7 @@ populate_identity_menu (Compose * mw, const char * id_name)
 	gint        index = 0;
 	GtkWidget * menu = gtk_menu_new ();
 	GPtrArray * identities = g_ptr_array_new ();
-	Identity  * id = NULL;
+	const Identity  * id = NULL;
 
 	/* get all identities */
 	identity_manager_get_identities (identities);
@@ -1271,26 +1533,32 @@ populate_identity_menu (Compose * mw, const char * id_name)
 	for (i=0; i<identities->len; ++i)
 	{
 		GtkWidget * item;
-		gchar     * label;
+		gchar buf[512];
+		gboolean have_addr;
+		gboolean have_name;
 
 		id = IDENTITY(g_ptr_array_index(identities,i));
 
 		/* get default id */
 		if (!pan_strcmp (id->name, id_name) || i == 0)
-		{
 			index = i;
-		}
 
-		label = g_strdup (id->name);
+		have_addr = is_nonempty_string (id->author_addr);
+		have_name = is_nonempty_string (id->author_real);
 
-		item = gtk_menu_item_new_with_label (label);
-		gtk_object_set_data_full (GTK_OBJECT(item), "identity_name",
-			g_strdup(id->name), g_free);
+	        if (have_addr && have_name)
+			g_snprintf (buf, sizeof(buf), "\"%s\" <%s>", id->author_real, id->author_addr);
+		else if (have_addr)
+			g_snprintf (buf, sizeof(buf), "%s", id->author_addr);
+		else if (have_name)
+			g_snprintf (buf, sizeof(buf), "%s", id->author_real);
+
+
+		item = gtk_menu_item_new_with_label (buf);
+		gtk_object_set_data_full (GTK_OBJECT(item), "identity_name", g_strdup(id->name), g_free);
 		gtk_signal_connect (GTK_OBJECT(item), "activate", (GtkSignalFunc)set_identity_cb, mw);
 		gtk_widget_show (item);
 		gtk_menu_append (GTK_MENU(menu), item);
-
-		g_free (label);
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(mw->from_om), menu);
@@ -1299,7 +1567,7 @@ populate_identity_menu (Compose * mw, const char * id_name)
 
 	/* remember the id name */
 	id = IDENTITY(g_ptr_array_index(identities, index));
-	replace_gstr(&mw->identity_name, g_strdup(id->name));
+	replace_gstr (&mw->identity_name, g_strdup(id->name));
 
 	pan_g_ptr_array_foreach (identities, (GFunc)pan_object_unref, NULL);
 	g_ptr_array_free (identities, TRUE);
@@ -1332,33 +1600,23 @@ populate_post_info_pane (Compose *mw)
 	**/
 
 	{
-		gchar * pch;
-		const gchar * cpch;
 		GString * gstr = g_string_new (NULL);
 
 		/* start with "X-Comment-To:" header for Fido users. */
-		pch = NULL;
-		if (a!=NULL)
-			pch = article_get_author_str (a);
-		if (!is_nonempty_string(pch))
-			pch = g_strdup ("ALL");
-		g_string_sprintfa (gstr, "X-Comment-To: %s\n", pch);
-		g_free (pch);
+		if (1) {
+			gchar author[512] = { '\0' };
+			article_get_author_str (a, author, sizeof(author));
+			g_string_sprintfa (gstr, "X-Comment-To: %s\n", *author ? author : "ALL");
+		}
 
-		/* add user's defaults */
-		cpch = extra_posting_headers;
-		g_string_append (gstr, cpch);
-
-		/* walk through all of the article's headers.
-		   If it's an extra one & wasn't covered in user's defaults add it */
+		/* add all the extra headers. */
 		if (a!=NULL && group_is_folder(a->group)) {
 			guint i;
 			GPtrArray * h = article_get_all_headers (a);
 			for (i=0; i!=h->len; i+=2) {
 				char* key = (char*)g_ptr_array_index(h,i);
 				char* val = (char*)g_ptr_array_index(h,i+1);
-				gboolean extra = article_header_is_extra (key);
-				if (extra && pan_strstr(pch,key)==NULL)
+				if (article_header_is_extra (key))
 					g_string_sprintfa (gstr, "%s: %s\n", key, val);
 			}
 			g_ptr_array_free (h, TRUE);
@@ -1409,19 +1667,19 @@ populate_post_info_pane (Compose *mw)
 
 		/* newsgroups */
 		g_assert (mw->newsgroups!=NULL);
-		header = article_get_header (a, HEADER_NEWSGROUPS);
+		header = article_get_extra_header (a, HEADER_NEWSGROUPS);
 		if (is_nonempty_string(header))
 			gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), header);
 
 		/* mail-to */
 		g_assert (mw->users!=NULL);
-		header = article_get_header (a, PAN_MAIL_TO);
+		header = article_get_extra_header (a, PAN_MAIL_TO);
 		if (is_nonempty_string(header))
 			gtk_entry_set_text (GTK_ENTRY(mw->users), header);
 
 		/* followup-to */
 		g_assert (mw->followup_to!=NULL);
-		header = article_get_header (a, HEADER_FOLLOWUP_TO);
+		header = article_get_extra_header (a, HEADER_FOLLOWUP_TO);
 		if (is_nonempty_string(header))
 			gtk_entry_set_text (GTK_ENTRY(mw->followup_to), header);
 
@@ -1458,18 +1716,20 @@ populate_post_info_pane (Compose *mw)
 		/* are we posting an article? */
 		if (a!=NULL && is_posting(mw->type))
 		{
-			const gchar * followup = article_get_header (a, HEADER_FOLLOWUP_TO);
-			const gchar * newsgroups = article_get_header (a, HEADER_NEWSGROUPS);
+			const gchar * followup = article_get_extra_header (a, HEADER_FOLLOWUP_TO);
+			const gchar * newsgroups = article_get_extra_header (a, HEADER_NEWSGROUPS);
 
 			/* if user set followup-to, use that;
 			 * otherwise, use newsgroups;
 			 * otherwise, use the currently-selected group */
 			if (is_nonempty_string(followup))
 			{
+				default_newsgroups = FALSE;
+
 				/* Followup-To: poster */
 				if (!g_strcasecmp ("poster", followup)) /* explicit reply by mail */
 				{
-					const gchar * replyto = article_get_header (a, HEADER_REPLY_TO);
+					const gchar * replyto = article_get_extra_header (a, HEADER_REPLY_TO);
 
 					gnome_ok_dialog_parented (
 						_("``Followup-To: poster'': sending email to author."),
@@ -1478,15 +1738,14 @@ populate_post_info_pane (Compose *mw)
 					if (is_nonempty_string(replyto))
 						gtk_entry_set_text (GTK_ENTRY(mw->users), replyto);
 					else {
-						gchar * pch = article_get_author_str (a);
-						gtk_entry_set_text (GTK_ENTRY(mw->users), pch);
-						g_free (pch);
+						gchar author[512];
+						article_get_author_str (a, author, sizeof(author));
+						gtk_entry_set_text (GTK_ENTRY(mw->users), author);
 					}
 				}
 				else /* explicit followup groups */
 				{
 					gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), followup);
-					default_newsgroups = FALSE;
 				}
 			}
 			else if (is_nonempty_string(newsgroups)) /* explicit groups */
@@ -1502,13 +1761,13 @@ populate_post_info_pane (Compose *mw)
 			/* figure out who to send mail to by checking reply-to: and from: */
 			if (a!=NULL)
 			{
-				const gchar * replyto = article_get_header (a, HEADER_REPLY_TO);
+				const gchar * replyto = article_get_extra_header (a, HEADER_REPLY_TO);
 				if (is_nonempty_string(replyto))
 					gtk_entry_set_text (GTK_ENTRY(mw->users), replyto);
 				else {
-					gchar * pch = article_get_author_str (a);
-					gtk_entry_set_text (GTK_ENTRY(mw->users), pch);
-					g_free (pch);
+					gchar author[512];
+					article_get_author_str (a, author, sizeof(author));
+					gtk_entry_set_text (GTK_ENTRY(mw->users), author);
 				}
 			}
 		}
@@ -1561,65 +1820,6 @@ populate_post_info_pane (Compose *mw)
 	}
 }
 
-/*
-static void
-attachments_update_lines (Compose *mw)
-{
-	struct stat statd;
-   	char *filename = gtk_entry_get_text (GTK_ENTRY(mw->filename));
-
-	if (filename && (0 == stat (filename, &statd))) {
-		char *p;
-		unsigned long lines;
-		unsigned long lines_per_part;
-
-		lines = statd.st_size / 45;
-		p = g_strdup_printf ("%lu", lines);
-		gtk_label_set_text (GTK_LABEL(mw->lines), p); 
-		g_free (p);
-		lines_per_part = gtk_spin_button_get_value_as_int (
-	                      GTK_SPIN_BUTTON(mw->lines_per_part));
-		p = g_strdup_printf ("%lu", (lines / lines_per_part) + 1);
-		gtk_label_set_text (GTK_LABEL(mw->parts), p); 
-		g_free (p);
-	}
-}
-*/
-
-/*
-static void
-attachments_lines_per_part_changed (GtkWidget *w, Compose *mw) {
-	attachments_update_lines (mw);
-}
-
-static void
-attachments_page_file_selected (GtkWidget *w, GtkWidget *f) {
-	char *selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(f));
-	Compose * mw = gtk_object_get_user_data (GTK_OBJECT(f));
-   	gtk_entry_set_text (GTK_ENTRY(mw->filename), selected_filename);
-	attachments_update_lines (mw);
-}
-
-static void
-attachment_file_selector (GtkWidget *menuitem, Compose * mw)
-{
-	GtkWidget *file_selector;
-	file_selector = gtk_file_selection_new(_("Select the file to attach."));
-   	gtk_object_set_user_data (GTK_OBJECT(file_selector), mw);
-
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-	                    "clicked", GTK_SIGNAL_FUNC (attachments_page_file_selected), 
-	                     file_selector);
-    	gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
-	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-	                           file_selector);
-	gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),
-	                           "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
-	                           file_selector);
-    
-    	gtk_widget_show (file_selector);
-}
-*/
 
 /* ctrl+return is GtkText's activate accelerator and Agent/Gravity's shortcut for "send now" */
 static void
@@ -1854,6 +2054,7 @@ message_window_new (Article * article, ComposeType type)
 		gtk_widget_grab_focus (mw->subject);
 		pan_unlock ();
 
+		gtk_window_set_default_size (GTK_WINDOW(mw->window), 525, 575);
 		gui_restore_window_size (mw->window, "message_window");
 
 		pan_lock ();
@@ -1986,9 +2187,9 @@ message_window_get_reverse_path (const Compose* mw)
 	Identity * id;
 	gchar    * from;
 	gchar    * retval = NULL;
-	gchar    * address;
-	gchar    * realname;
 	int        status;
+	gchar      address[512];
+	gchar      realname[512];
 
 	g_return_val_if_fail (mw!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string (mw->identity_name), NULL);
@@ -1997,15 +2198,14 @@ message_window_get_reverse_path (const Compose* mw)
 	from = g_strdup_printf ("\"%s\" <%s>", id->author_real, id->author_addr);
 	pan_object_unref (PAN_OBJECT(id));
 
-	status = gnksa_do_check_from (from, &address, &realname, FALSE);
-
+	status = gnksa_do_check_from (from,
+	                              address, sizeof(address),
+	                              realname, sizeof(realname),
+				      FALSE);
 
 	if (status == GNKSA_OK)
-		retval = address;
-	else
-		g_free (address);
+		retval = g_strdup (address);
 
-	g_free (realname);
 	return retval;
 }
 
@@ -2021,6 +2221,9 @@ populate_article_from_mw (Article * article, const Compose * mw)
 	g_return_if_fail (mw!=NULL);
 	g_return_if_fail (is_nonempty_string (mw->identity_name));
 
+	/* identity */
+	id = identity_manager_get_identity (mw->identity_name);
+
 	/* date */
 	article->date = time(0);
 
@@ -2028,17 +2231,18 @@ populate_article_from_mw (Article * article, const Compose * mw)
 	article->number = 1;
 
 	/* author */
-	id = identity_manager_get_identity (mw->identity_name);
 	article->author_addr = is_nonempty_string (id->author_addr)
 		? group_chunk_string (article->group, id->author_addr, TRUE)
 		: NULL;
 	article->author_real = is_nonempty_string (id->author_real)
 		? group_chunk_string (article->group, id->author_real, TRUE)
 		: NULL;
-	pan_object_unref(PAN_OBJECT(id));
 
 	/* message-id */
-	pch = gnksa_generate_message_id_from_email_addr (article->author_addr);
+	if (is_nonempty_string (id->msg_id_fqdn)) 
+		pch = gnksa_generate_message_id (id->msg_id_fqdn);
+	else
+		pch = gnksa_generate_message_id_from_email_addr (article->author_addr);
 	article_init_header (article, HEADER_MESSAGE_ID, pch, DO_CHUNK);
 	g_free (pch);
 
@@ -2051,7 +2255,7 @@ populate_article_from_mw (Article * article, const Compose * mw)
 	if ((mw->type==NNTP_REPLY || mw->type==EMAIL_REPLY || mw->type==EMAIL_AND_POST_REPLY)
 	    && (mw->article != NULL))
 	{
-		const gchar * refs = article_get_header (mw->article, HEADER_REFERENCES);
+		const gchar * refs = mw->article->references;
 		const gchar * msg_id = article_get_message_id (mw->article);
 		pch = gnksa_generate_references (refs, msg_id);
 		if (pch != NULL) {
@@ -2142,6 +2346,9 @@ populate_article_from_mw (Article * article, const Compose * mw)
 	}
 	g_strfreev (sd);
 	g_free (pch);
+
+	/* cleanup */
+	pan_object_unref(PAN_OBJECT(id));
 }
 
 static Article*
@@ -2239,7 +2446,7 @@ post (Compose * mw, SendMode mode)
 
 		/* store which server to post through, in case the article
 		 * winds up waiting in pan.sendlater */
-		if (!is_nonempty_string(article_get_header (article, PAN_SERVER)))
+		if (!is_nonempty_string(article_get_extra_header (article, PAN_SERVER)))
 			article_set_header (article, PAN_SERVER, server->name, DO_CHUNK_SHARE);
 
 		/* add this article to the sendlater group */
@@ -2250,7 +2457,7 @@ post (Compose * mw, SendMode mode)
 			queue_article_for_posting (server, article, FALSE);
 
 		/* remember which identity we used */
-		pch = article_get_header (article, HEADER_NEWSGROUPS);
+		pch = article_get_extra_header (article, HEADER_NEWSGROUPS);
 		while ((group_name = get_next_token_str (pch, ',', &pch))!=NULL)
 		{
 			Group * group;
@@ -2442,7 +2649,7 @@ compose_input_cb (gpointer data, gint source, GdkInputCondition condition)
 
 	close (source);
 
-	gtk_widget_set_sensitive (mw->body, TRUE);
+	gtk_widget_set_sensitive (mw->window, TRUE);
 
 	replace_gstr (&mw->exteditor_file, NULL);
 	mw->exteditor_pid = -1;
@@ -2512,7 +2719,7 @@ compose_exec_ext_editor_real (const gchar * file)
 		perror ("setpgid");
 
 	tmp = pan_substitute (external_editor, "%t", file);
-	cmdline = g_strsplit (tmp, " ", 1024);
+	cmdline = strsplit_command_line_arguments (tmp);
 	execvp (cmdline[0], cmdline);
 
 	perror ("execvp");
@@ -2530,10 +2737,12 @@ compose_exec_ext_editor (Compose * mw)
 	pid_t pid;
 	gint pipe_fds[2];
 
-	g_snprintf (tmp, sizeof(tmp), "%s%ctmpmsg.%08x",
-	            g_get_tmp_dir(),
-	            G_DIR_SEPARATOR,
-	            GPOINTER_TO_INT(mw));
+	/* get a temporary filename */
+	if (1) {
+		gchar * tmp_dir = pan_substitute (g_get_tmp_dir(), "~", g_get_home_dir());
+		g_snprintf (tmp, sizeof(tmp), "%s%ctmpmsg.%08x", tmp_dir, G_DIR_SEPARATOR, GPOINTER_TO_INT(mw));
+		g_free (tmp_dir);
+	}
 
 	if (pipe (pipe_fds) < 0) {
 		perror ("pipe");
@@ -2554,7 +2763,7 @@ compose_exec_ext_editor (Compose * mw)
 		mw->exteditor_pid = pid;
 		mw->exteditor_readdes = pipe_fds[0];
 
-		gtk_widget_set_sensitive (mw->body, FALSE);
+		gtk_widget_set_sensitive (mw->window, FALSE);
 
 		mw->exteditor_tag = gdk_input_add (pipe_fds[0], GDK_INPUT_READ, compose_input_cb, mw);
 	}
