@@ -45,6 +45,7 @@ extern ListClickAction         grouplist_mb1_action;
 extern ListClickAction         grouplist_mb2_action;
 extern ListClickAction         articlelist_mb1_action;
 extern ListClickAction         articlelist_mb2_action;
+extern gboolean                header_pane_is_threaded;
 extern gboolean                show_group_pane;
 extern gboolean                show_header_pane;
 extern gboolean                show_article_pane;
@@ -56,6 +57,7 @@ extern gboolean                text_window_smooth_scrolling;
 extern gint                    text_window_smooth_scrolling_speed;
 extern gboolean                fetch_new_on_group_load;
 extern gboolean                display_article_dates_in_local_time;
+extern gboolean                expand_all_threads_by_default;
 extern gboolean                hide_mpart_child_nodes;
 extern gboolean                auto_part_selection;
 extern gboolean                pan_mute;
@@ -70,7 +72,6 @@ extern gchar                 * message_body_font_fixed;
 extern gchar                 * external_web_browser;
 extern gchar                 * external_editor;
 extern gint                    watched_thread_span_days;
-extern gchar                 * extra_posting_headers;
 extern gchar                 * mail_server_address;
 extern gchar                 * layout_str;
 extern gint                    mail_server_port;
