@@ -99,16 +99,13 @@ SocketInfo;
 **********************  Private Function Prototypes
 *********************/
 
-static void queue_mainloop (void);
+static void* queue_mainloop (void*);
 
 static void queue_run_what_we_can (void);
 
 static void queue_set_task_status (Task* task,
                                    QueueTaskStatus status);
 
-static int sockets_find_socket (gconstpointer a,
-                                gconstpointer b);
-
 static void socket_cleanup (Task* task);
 
 static void queue_new_todo (QueueTodoAction action,
@@ -116,7 +113,7 @@ static void queue_new_todo (QueueTodoAction action,
                             int i1,
                             int i2);
 
-static int queue_get_length (void);
+static guint queue_get_length (void);
 
 static void queue_do_todo (void);
 
@@ -158,15 +155,15 @@ PanCallback * queue_max_tries_changed    = NULL;
 ************  Private
 ***********/
 
-static pthread_mutex_t qlock;
 
 /* FIXME glib 2.0: for efficiency of adding to end, use a GQueue instead of GSList */
 static GSList* todo_list = NULL;
-static GSList* queue_list = NULL;
+static GSList* task_list = NULL;
 
 static pthread_mutex_t sock_lock;
 static pthread_mutex_t cond_mutex;
 static pthread_mutex_t todo_mutex;
+static pthread_mutex_t task_lock;
 static pthread_cond_t qcond = PTHREAD_COND_INITIALIZER;
 static gboolean work_to_do = FALSE;
 
@@ -180,7 +177,7 @@ static int max_socket_qty = 4;
 
 static int max_tries = 5;
 
-static int running_tasks = 0;
+static guint running_tasks = 0;
 
 static gboolean paused = FALSE;
 
@@ -227,6 +224,7 @@ queue_init (int socket_qty, int tries)
 	pthread_mutex_init (&cond_mutex, NULL);
 	pthread_mutex_init (&todo_mutex, NULL);
 	pthread_mutex_init (&sock_lock, NULL);
+	pthread_mutex_init (&task_lock, NULL);
 
 	pan_callback_add (server_get_online_status_changed_callback(),
 	                  server_online_status_changed_cb, NULL);
@@ -235,7 +233,7 @@ queue_init (int socket_qty, int tries)
 	max_socket_qty = socket_qty;
 
 	/* fire up the queue thread */
-	pthread_create (&queue_thread, NULL, (void*)queue_mainloop, NULL);
+	pthread_create (&queue_thread, NULL, queue_mainloop, NULL);
 	pthread_detach (queue_thread);
 
 	debug_exit ("queue_init");
@@ -258,12 +256,15 @@ real_queue_set_paused (gint p)
 	if ((paused = p))
 	{
 		GSList * l;
+
+		pthread_mutex_lock (&sock_lock);
 		for (l=sockets; l!=NULL; l=l->next)
 		{
 			SocketInfo * i = (SocketInfo*)l->data;
 			task_hint_abort (i->task);
 			queue_set_task_status (i->task, QUEUE_TASK_STATUS_ABORTING);
 		}
+		pthread_mutex_unlock (&sock_lock);
 	}
 }
 
@@ -297,9 +298,13 @@ real_queue_shutdown (int save_tasks)
 
 	/* unref the tasks */
 
-	/* close the sockets */
+	/* get the socket array */
+	pthread_mutex_lock (&sock_lock);
 	sockets_tmp = sockets;
 	sockets = NULL;
+	pthread_mutex_unlock (&sock_lock);
+
+	/* close the sockets */
 	len = g_slist_length (sockets_tmp);
 	for (l=sockets_tmp; l!=NULL; l=l->next)
 		pan_object_unref (PAN_OBJECT(((SocketInfo*)l->data)->socket));
@@ -322,12 +327,12 @@ queue_get_tasks (void)
 	GSList * l;
 	GPtrArray * a = g_ptr_array_new ();
 
-        pthread_mutex_lock (&todo_mutex);
-	for (l=queue_list; l!=NULL; l=l->next) {
+        pthread_mutex_lock (&task_lock);
+	for (l=task_list; l!=NULL; l=l->next) {
 		pan_object_ref (PAN_OBJECT(l->data));
 		g_ptr_array_add (a, l->data);
 	}
-        pthread_mutex_unlock (&todo_mutex);
+        pthread_mutex_unlock (&task_lock);
 
 	return a;
 }
@@ -360,17 +365,17 @@ queue_get_running_task_count (void)
 	gint running_count = 0;
 	GSList *l;
 
-	for (l=queue_list; l!=NULL; l=l->next)
+	pthread_mutex_lock (&task_lock);
+	for (l=task_list; l!=NULL; l=l->next)
 	{
 		QueueTaskStatus status = queue_get_task_status (l->data);
 
 		if (status == QUEUE_TASK_STATUS_RUNNING ||
 		    status == QUEUE_TASK_STATUS_QUEUED ||
 		    status == QUEUE_TASK_STATUS_CONNECTING)
-		{
 			++running_count;
-		}
 	}
+	pthread_mutex_unlock (&task_lock);
 
 	return running_count;
 }
@@ -425,22 +430,23 @@ queue_task_insert (Task* task, int index)
 static void
 real_queue_task_insert (Task* task, int index)
 {
+	guint task_len;
 	debug2 (DEBUG_QUEUE, "queue_task_insert %p to pos %d", task, index);
 
 	g_return_if_fail (task!=NULL);
-	g_return_if_fail (index>=-1);
-	g_return_if_fail (index<=(int)queue_get_length());
+	g_return_if_fail (index==-1 || (guint)index<=queue_get_length());
 
 	/* add to the list */
-	pthread_mutex_lock (&qlock);
-	queue_list = g_slist_insert (queue_list, task, index);
-	g_hash_table_insert (task_to_status, task,
-		GINT_TO_POINTER(QUEUE_TASK_STATUS_QUEUED));
-	pthread_mutex_unlock (&qlock);
-	gui_set_queue_size (running_tasks, (int)g_slist_length(queue_list));
+	pthread_mutex_lock (&task_lock);
+	task_list = g_slist_insert (task_list, task, index);
+	g_hash_table_insert (task_to_status, task, GINT_TO_POINTER(QUEUE_TASK_STATUS_QUEUED));
+	task_len = g_slist_length (task_list);
+	pthread_mutex_unlock (&task_lock);
+
+	gui_set_queue_size (running_tasks, task_len);
 
 	/* call callbacks */
-	debug1 (DEBUG_QUEUE, "queue size is now %d, calling callbacks", queue_get_length());
+	debug1 (DEBUG_QUEUE, "queue size is now %u, calling callbacks", task_len);
 	pan_callback_call (queue_task_added, task, GINT_TO_POINTER(index));
 
 	queue_wakeup ();
@@ -470,15 +476,19 @@ real_queue_task_remove (Task* task)
 	}
 	else /* not running; simple cleanup. queue_run_thread() zeroes out is_running */
 	{
-		const int old_index = g_slist_index (queue_list, task);
+		gint old_index;
+		guint task_len;
 		const gboolean had_socket = task->sock != NULL;
 		socket_cleanup (task);
 
 		/* remove the task */
-		pthread_mutex_lock (&qlock);
-		queue_list = g_slist_remove (queue_list, task);
-		pthread_mutex_unlock (&qlock);
-		gui_set_queue_size (running_tasks, (int)g_slist_length(queue_list));
+		pthread_mutex_lock (&task_lock);
+	       	old_index = g_slist_index (task_list, task);
+		task_list = g_slist_remove (task_list, task);
+		task_len = g_slist_length (task_list);
+		pthread_mutex_unlock (&task_lock);
+
+		gui_set_queue_size (running_tasks, task_len);
 
 		/* fire callbacks */
 		pan_callback_call (queue_task_removed, task, GINT_TO_POINTER(old_index));
@@ -512,27 +522,24 @@ real_queue_task_move (Task* task, int index)
 
 	/* sanity clause */
 	g_return_if_fail (task!=NULL);
-	g_return_if_fail (index>=-1);
-	g_return_if_fail (index<=(int)queue_get_length());
+	g_return_if_fail (index==-1 || (guint)index<=queue_get_length());
 	
-	pthread_mutex_lock (&qlock);
+	pthread_mutex_lock (&task_lock);
 	
-	old_index = g_slist_index (queue_list, task);
-	g_return_if_fail (old_index!=-1);
+	old_index = g_slist_index (task_list, task);
+	if (old_index != -1)
+	{
+		task_list = g_slist_remove (task_list, task);
+		task_list = g_slist_insert (task_list, task, index);
+		
+		indices[0] = old_index;
+		indices[1] = index;
+	}
 
-	queue_list = g_slist_remove (queue_list, task);
-	queue_list = g_slist_insert (queue_list, task, index);
-	
-	indices[0] = old_index;
-	indices[1] = index;
+	pthread_mutex_unlock (&task_lock);
 
-	pthread_mutex_unlock (&qlock);
-
-	debug3 (DEBUG_QUEUE,
-                "queue_task_move %p calling callbacks; "
-                "old index is %d, new index is %d",
-                task, old_index, index);
-	pan_callback_call (queue_task_moved, task, indices);
+	if (old_index != -1)
+		pan_callback_call (queue_task_moved, task, indices);
 
 	debug1 (DEBUG_QUEUE, "queue_task_move %p done", task);
 }
@@ -601,7 +608,7 @@ static void
 queue_set_task_status (Task* task, QueueTaskStatus status)
 {
 	g_return_if_fail (task!=NULL);
-	g_return_if_fail (g_slist_index(queue_list, task) != -1);
+	g_return_if_fail (g_slist_index(task_list, task) != -1);
 
 	g_hash_table_insert (task_to_status, task,
 	                     GINT_TO_POINTER(status));
@@ -616,7 +623,7 @@ queue_set_task_status (Task* task, QueueTaskStatus status)
 static void
 queue_new_todo (QueueTodoAction action, Task* task, int i1, int i2)
 {
-	QueueTodo *todo = g_new0 (QueueTodo, 1);
+	QueueTodo *todo = g_new (QueueTodo, 1);
 	todo->task = task;
 	todo->index_1 = i1;
 	todo->index_2 = i2;
@@ -631,25 +638,17 @@ queue_new_todo (QueueTodoAction action, Task* task, int i1, int i2)
 	queue_wakeup ();
 }
 
-static int
+static guint
 queue_get_length (void)
 {
-	int size = -1;
-	pthread_mutex_lock (&qlock);
-	size = g_slist_length (queue_list);
-	pthread_mutex_unlock (&qlock);
+	guint size;
+	pthread_mutex_lock (&task_lock);
+	size = g_slist_length (task_list);
+	pthread_mutex_unlock (&task_lock);
 	return size;
 }
 
-static int
-sockets_find_socket (gconstpointer a, gconstpointer b)
-{
-	const SocketInfo * info = (const SocketInfo*)a;
-	const PanSocket * socket = (const PanSocket*)b;
-	return info->socket != socket;
-}
-
-static void
+static void*
 queue_run_thread (void* vp)
 {
 	/* preparing to run */
@@ -663,20 +662,23 @@ queue_run_thread (void* vp)
 	status_item_set_active (STATUS_ITEM(task), TRUE);
 	if (task->sock)
 		pan_socket_reset_statistics (task->sock);
-	gui_set_queue_size (++running_tasks, (int)g_slist_length(queue_list));
+	gui_set_queue_size (++running_tasks, queue_get_length());
 	result = task_run (task);
-	gui_set_queue_size (--running_tasks, (int)g_slist_length(queue_list));
+	gui_set_queue_size (--running_tasks, queue_get_length());
 	debug3 (DEBUG_QUEUE, "queue task[%p] (%s) returned: %d", task, desc, result);
 	g_free(desc);
 
 	/* clean up gui */
 	status_item_set_active (STATUS_ITEM(task), FALSE);
 
-	/* clean up socket */
+	/* clean up socket.
+	   We throw away any socket of a failed or aborted task,
+	   just in case the read buffer has junk in there. */
 	aborted = queue_get_task_status(task) == QUEUE_TASK_STATUS_ABORTING;
-	if (aborted)
+	if (task->sock!=NULL && (aborted || result!=TASK_SUCCESS))
 		task->sock->error = TRUE;
 	socket_cleanup (task);
+
 	/* clean up the task */
 	task->is_running = FALSE;
 
@@ -700,10 +702,12 @@ queue_run_thread (void* vp)
 		task->tries = 0;
 		task->hint_abort = FALSE;
 		queue_set_task_status (task, QUEUE_TASK_STATUS_FAILED);
+		queue_wakeup ();
 	}
 
 	debug1 (DEBUG_QUEUE, "queue_run_thread: task %p done", vp);
 	pthread_exit (NULL);
+	return NULL;
 }
 
 
@@ -717,7 +721,7 @@ queue_run (Task* task)
 	queue_set_task_status (task, QUEUE_TASK_STATUS_RUNNING);
 
 	task->is_running = TRUE;
-	pthread_create (&thread, NULL, (void*)queue_run_thread, task);
+	pthread_create (&thread, NULL, queue_run_thread, task);
 	pthread_detach (thread);
 }
 
@@ -864,7 +868,7 @@ queue_get_socket_for_task (Task *task)
 				debug1 (DEBUG_QUEUE, "%p logged in", sock);
 
 				/* looks like we got a new socket okay... */
-				si = g_new0 (SocketInfo, 1);
+				si = g_new (SocketInfo, 1);
 				si->socket = sock;
 				si->server = server;
 				si->task = task;
@@ -885,18 +889,17 @@ queue_get_socket_for_task (Task *task)
 }
 
 /**
- * Run any tasks in the queue_list which can be run (ie, can get a socket,
+ * Run any tasks in the task_list which can be run (ie, can get a socket,
  * if necessary) right now.
  */
 static void
 queue_run_what_we_can (void)
 {
-	GSList *l = NULL;
+	GSList * l = NULL;
 	debug_enter ("queue_run_what_we_can");
 
-	/* walk through the queue and see if
-	   any tasks are waiting to be run. */
-	l = queue_list;
+	/* walk through the queue and see if any tasks are waiting to be run. */
+	l = task_list;
 	while (l!=NULL)
 	{
 		Task *task = TASK(l->data);
@@ -936,6 +939,9 @@ queue_run_what_we_can (void)
 			{
 				if (i==-2) /* connection failure */
 				{
+
+
+
 					status_item_emit_error (STATUS_ITEM(task), _("Connect Failure"));
 
 					++task->tries;
@@ -947,10 +953,10 @@ queue_run_what_we_can (void)
 			}
 		}
 
-		if (todo_list)
+		if (todo_list != NULL)
 		{
 				queue_do_todo ();
-				l = queue_list;
+				l = task_list;
 		}
 		else
 		{
@@ -976,19 +982,24 @@ socket_cleanup (Task* task)
 		GSList* l = NULL;
 		SocketInfo * info = NULL;
 		int server_socket_qty;
+		int socket_qty;
 
 		pthread_mutex_lock (&sock_lock);
 
 		/* count the number of sockets this server has */
+		socket_qty = 0;
 		server_socket_qty = 0;
-		for (l=sockets; l!=NULL; l=l->next)
-			if (((SocketInfo*)l->data)->server == task->server)
+		for (l=sockets; l!=NULL; l=l->next) {
+			SocketInfo * sockinfo = (SocketInfo*) l->data;
+			if (sockinfo->socket == socket)
+				info = sockinfo;
+			if (sockinfo->server == task->server)
 				++server_socket_qty;
+			++socket_qty;
+		}
 
 		/* find this SocketInfo in our tables */
-		l = g_slist_find_custom (sockets, socket, sockets_find_socket);
-		g_assert (l!=NULL);
-		info = (SocketInfo*) l->data;
+		g_assert (info!=NULL);
 
 		/**
                  *  Recycle the socket if all are true
@@ -999,21 +1010,18 @@ socket_cleanup (Task* task)
 		 */
 		if (!socket->error
 		    && task->server->online
-		    && (int)g_slist_length(sockets)<=max_socket_qty
+		    && socket_qty<=max_socket_qty
 		    && server_socket_qty<=task->server->max_connections)
 		{
 		       	/* if socket's okay, reuse it */
 			info->task = NULL;
 
-			debug1 (DEBUG_QUEUE,
-				"socket_cleanup: recycling socket %p",
-				socket);
+			debug1 (DEBUG_QUEUE, "socket_cleanup: recycling socket %p", socket);
 		}
 		else
 		{
 			/* otherwise throw it away */
-			pan_socket_putline (socket, "ccc\r\n");
-			nntp_disconnect (NULL, socket);
+			pan_socket_putline (socket, "QUIT\r\n");
 			close (socket->sockfd);
 			socket->sockfd = 0;
 			pan_object_unref (PAN_OBJECT(socket));
@@ -1025,12 +1033,9 @@ socket_cleanup (Task* task)
 				"socket_cleanup: throwing away socket %p",
 				socket);
 		}
-
 		task->sock = NULL;
-
 		pthread_mutex_unlock (&sock_lock);
 	}
-
 	debug_exit ("socket_cleanup");
 }
 
@@ -1108,8 +1113,7 @@ sockets_upkeep (void)
 		}
 	}
 
-	debug1 (DEBUG_QUEUE,
-		"sockets_upkeep: culling %d sockets",
+	debug1 (DEBUG_QUEUE, "sockets_upkeep: culling %u sockets",
 		g_slist_length(cull));
 
 	for (l=cull; l!=NULL; l=l->next)
@@ -1122,7 +1126,8 @@ sockets_upkeep (void)
 
 		/* politely disconnect */
 		status_item_set_active (status, TRUE);
-		nntp_disconnect (status, sock);
+		if (!sock->error)
+			nntp_disconnect (status, sock);
 		status_item_set_active (status, FALSE);
 		log_add_va (LOG_INFO, _("Closing connection %p after %d seconds idle"), sock, age);
 
@@ -1207,10 +1212,10 @@ queue_do_todo (void)
 	list = NULL;
 }
 
-static void
-queue_mainloop (void)
+static void*
+queue_mainloop (void* unused)
 {
-	const int TIMEOUT_SECS = 15;
+	const int TIMEOUT_SECS = 60;
 
 	for (;;)
 	{
@@ -1236,4 +1241,6 @@ queue_mainloop (void)
                 	queue_run_what_we_can ();
 		}
 	}
+
+	return NULL;
 }
