@@ -125,12 +125,19 @@ save_attachment_as_dialog_clicked_cb (GnomeDialog      * dialog,
 	**/
 
 	path = NULL;
-	if (IS_ACTIVE(save->user_specified_rb))
+	if (IS_ACTIVE(save->user_specified_rb)) {
 		path = gtk_editable_get_chars (GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY (save->user_specified_entry))), 0, -1);
-	else if ((save->group_default_dir_rb!=NULL) && IS_ACTIVE(save->group_default_dir_rb))
+		g_strstrip (path);
+		if (path!=NULL && *path=='~')
+			replace_gstr (&path, g_strconcat (g_get_home_dir(), path+1, NULL));
+	}
+	else if ((save->group_default_dir_rb!=NULL) && IS_ACTIVE(save->group_default_dir_rb)) {
 		path = g_strdup(save->group->download_dir);
-	else
+	}
+	else {
 		path = g_strdup (download_dir);
+	}
+
 	if (IS_ACTIVE(save->append_groupname_tb))
 		replace_gstr (&path, g_strdup_printf ("%s%c%s", path, G_DIR_SEPARATOR, save->group->name));
 
@@ -180,11 +187,11 @@ save_attachment_as_dialog_clicked_cb (GnomeDialog      * dialog,
 		if (save_bodies)
 		{
 			const gchar * filename = NULL;
-			if (use_custom)
+			if (use_custom_for_body)
 				filename = custom_body_fname;
 			else if (use_message_id)
 				filename = article_get_message_id (article);
-			else if (use_subject)
+			else if (use_subject_for_body)
 				filename = subject;
 			task_save_set_bodies (TASK_SAVE(task), path, filename);
 		}
