@@ -15,6 +15,8 @@
 			g_message ("[%d][%s]", i, (gchar*)g_ptr_array_index(errors,i)); \
 	}
 
+extern int server_saving_enabled;
+
 int main (void)
 {
 	gint test = 0;
@@ -27,6 +29,7 @@ int main (void)
 
 	/* init panbase */
 	g_thread_init (NULL);
+	server_saving_enabled = FALSE;
 	if (1) {
 		gchar * data_dir = g_strdup_printf ("%s/data/", getenv("PWD"));
 		gchar * download_dir = g_strdup_printf ("%s/data/save/", getenv("PWD"));
@@ -39,6 +42,7 @@ int main (void)
 	s = server_new ();
 	s->name = g_strdup ("Server");
 	g = group_new (s, "alt.test");
+	group_ref_articles (g, NULL);
 	server_add_groups (s, &g, 1, NULL, NULL);
 	a = article_new (g);
 
